/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe abstrata de Comandos - Descrio na prpria classe.
 ************************************************************************/

namespace ERWon
{
	/// <summary>
	/// Toda ao que possa ser desfeita  encapsulada em um
	/// objeto de comando. As classes derivadas provem mtodos de fazer/desfazer
	/// suas aes. Command  uma classe abstrata com os mtodos e propriedades
	/// necessrias para que suas classes derivadas sejam usadas pelo CommandProcessor.
	/// </summary>
	public abstract class Command
	{
		/// <summary>Comando pode ser desfeito? Usado pelo CommandProcessor.</summary>
		protected bool m_fCanUndo = false; // Nem todo comando possuir undo em um primeiro momento, posteriormente a idia  implementar em todos comandos.
		public bool CanUndo { get { return m_fCanUndo; } }

		/// <summary>Descrio do comando, para aparecer nos menus.</summary>
		/// Inicialmente eu havia colocado no construtor, mas as vezes o texto  varivel (depende do contexto - objetos selecionados, etc)
		protected string m_strShortDescription;
		public virtual string ShortDescription { get { return m_strShortDescription; } }

		/// <summary>Documento ao qual o comando se refere</summary>
		/// Normalmente refere-se ao (atual) documento-pai do Box ou Line do comando.
		public virtual Document ParentDocument { get { return null; } }


		/// <summary>Mtodo abstrato para executar o comando</summary>
		///  chamado tanto na primeira vez quanto nas prximas vezes,
		/// se o usurio desfizer e refize-lo.
		public abstract void Execute ();

		/// <summary>Mtodo abstrato para desfazer o comando</summary>
		public abstract void Undo ();
	}
}
