/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Aqui  o UserControl responsvel pela maior parte das funes...
 *  ele que  instanciado dentro dos MDI Windows.
 * Responsvel pelo OnPaint, tratamento de scroll, zoom, clicks, drag-drops, etc etc.
 ************************************************************************/

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

namespace ERWon
{

	/// <summary>
	/// Este  o Usercontrol que ficar nas janelas MDI child.
	/// Permite arrastar os objetos, redimensionar, controle de scroll, zoom, etc.
	/// </summary>
	public class Control : System.Windows.Forms.UserControl
	{
		private System.ComponentModel.IContainer components;

		/// <summary>Interface de Callback - Para chamar funes do MainForm a partir do controle.</summary>
		IControlCallbacks m_Callback;

		/// <summary>O documento em si: Lista de elementos, propriedades, etc</summary>
		IDocument m_Document;

		/// <summary>O Box atualmente selecionado (para arrastar ou redimensionar)</summary>
		IBox m_PickedBox;

		/// <summary>A linha que foi selecionada (para ser arrastada)</summary>
		ILine m_PickedLine;


		// Flags de controle... (redimensionando elemento, movendo elemento, e teclas pressionadas)
		/// <summary>Redimensionando elemento?</summary>
		bool m_fResizing;
		/// <summary>Arrastando (movendo) elemento?</summary>
		bool m_fMoving = false;


		/// At eu descobrir o Control.ModifierKeys (que retorna teclas shift, ctrl, etc)
		/// eu monitorava (nos eventos keydown e keyup) as mudanas destas teclas
		/// e armazenava seu estado. Pra variar, perda de tempo...
		//bool m_fControlKey = false;
		//bool m_fShiftKey = false;

		/// <summary>Este bitmat  usado para fazer um buffer das operaes de desenho
		/// para evitar flickering.</summary>
		//		System.Drawing.Bitmap m_BackBuffer = new System.Drawing.Bitmap (Screen.PrimaryScreen.Bounds.Right, Screen.PrimaryScreen.Bounds.Bottom);
		// No estou mais usando... o proprio .NET tem suporte nativo a double-buffering.

		/// <summary>Armazena os extremos do documento, para controle de Scroll.</summary>
		/// Inicialmente eu havia feito toda a lgica de scrolls na mo (com os objetos Scrollbar),
		/// at descobrir que o suporte a scroll  nativo... :((
		Rectangle m_DocumentBounds = new Rectangle ();

		/// <summary>Zoom do controle. Em porcentagem (1=100%)</summary>
		float _zoom=1.0f;


		/// <summary>Construtor</summary>
		/// <param name="Callback">Form principal do programa</param>
		/// <param name="Document">Interface do documento a ser mostrado no controle</param>
		public Control (IControlCallbacks Callback, IDocument Document)
		{
			InitializeComponent();

			// Para evitar flickering... perdi tanto tempo a toa... :(
			this.SetStyle(ControlStyles.AllPaintingInWmPaint|
				ControlStyles.UserPaint|
				ControlStyles.DoubleBuffer,true);

			BackColor = System.Drawing.Color.White; // cor padro

			m_Callback = Callback;
			m_Document = Document;
		}

		
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}



		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// Control
			// 
			this.AutoScroll = true;
			this.ForeColor = System.Drawing.SystemColors.Window;
			this.Name = "Control";
			this.Size = new System.Drawing.Size(150, 112);
			this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.OnMouseUp);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.OnPaint);
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.OnKeyDown);
			this.DoubleClick += new System.EventHandler(this.Control_DoubleClick);
			this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.OnMouseMove);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.OnMouseDown);
		}
		#endregion

		
		/// <summary>Determina o objeto selecionado nas coordenadas do mouse</summary>
		/// <param name="List">Lista de objetos para testar (Boxes ou Lines)</param>
		/// <param name="e">Coordenadas do mouse</param>
		IDocumentElement GetPicked (ArrayList List, Point pt)
		{
			// Vamos buscar do mais recente para o mais antigo, pois podem haver elementos sobrepostos e o mais recente est no topo
			// ArrayList ReverseList = (ArrayList) List.Clone();
			//ReverseList.Reverse();
			//foreach (IDocumentElement element in ReverseList)
			for (int i=List.Count-1; i>=0; i--)
			{
				IDocumentElement element = (IDocumentElement) List[i];

				if (element.Hit (pt.X, pt.Y))
					return element;
			}
			return null;
		}
		IDocumentElement GetPicked (ArrayList List, System.Windows.Forms.MouseEventArgs e) 
		{
			Point pt = RelativeMousePosition(e.X, e.Y);
			return GetPicked(List, pt);
		}


		
		/// <summary>Arrasta (Move) todos objetos selecionados. S  usado no tratamento de setas do teclado</summary>
		/// TODO: implementar em classe Command(), para poder desfazer ao.
		void DragSelected (int iDeltaX, int iDeltaY)
		{
			foreach (IBox ThisBox in m_Document.Boxes)
			{
				if (((DocElement) ThisBox).Selected) 
				{
					ThisBox.X0 += iDeltaX;
					ThisBox.X1 += iDeltaX;
					ThisBox.Y0 += iDeltaY;
					ThisBox.Y1 += iDeltaY;
				}
			}			
		}


		
		/// <summary>Redesenha o documento</summary>
		void OnPaint (object Sender, PaintEventArgs Args)
		{
			/// Eu estava usando um backbuffer pra evitar flickering, at constatar
			/// que j existe suporte nativo pra isso. :(  - Ver o this.SetStyle l em cima. 
			//Graphics OffScreen = Graphics.FromImage (m_BackBuffer);
			//OffScreen.Clear(Color.White);

			Update_Scrollbars(); // calcula o tamanho das scrollbars

			// Limpar o background
			Args.Graphics.Clear(m_Document.Background);

			// Aplicando Zoom via transformaes matriciais.
			System.Drawing.Drawing2D.Matrix mx=new System.Drawing.Drawing2D.Matrix();
			mx.Scale(_zoom, _zoom, System.Drawing.Drawing2D.MatrixOrder.Append);
			Args.Graphics.Transform=mx;

			// Aplicando translao baseada na posio das barras de scroll
			Point pt = this.AutoScrollPosition;
			Args.Graphics.TranslateTransform(pt.X / _zoom, pt.Y / _zoom);
			
			// Desenhar as Lines
			foreach (ILine ThisLine in m_Document.Lines)
				ThisLine.Draw (Args.Graphics);

			// Desenhar os Boxes
			foreach (IBox ThisBox in m_Document.Boxes)
				ThisBox.Draw (Args.Graphics);

			// Desenhar o documento
			(m_Document as Document).Draw(Args.Graphics, this);

			//Args.Graphics.DrawImage (m_BackBuffer, 0, 0);
		}


		/// <summary>
		/// Se o usurio clicar no mouse, pode ser:
		/// - Selecionando um Box
		/// - Selecionando um Line
		/// - Redimensionando Box
		/// - Movendo Box
		/// - Movendo Line
		/// </summary>
		void OnMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == System.Windows.Forms.MouseButtons.Left)
			{
				// Dependendo da ferramenta selecionada na barra de ferramentas, faa o que tiver que ser feito...
				// TODO - isso poderia estar melhor desenhado, cada boto cuidaria do seu evento, etc.
				switch(m_Callback.BotaoSelecionado())
				{
					case botao_toolbar.Add_Entity:
						Click_AddEntity(e);
						m_Callback.Toolbar_Reset();
						break;
					case botao_toolbar.SelectionTool:
						Click_SelectionTool(e);
						break;
					case botao_toolbar.Add_NonIdentifying_Relationship:
						Click_Add_NonIdentifying_Relationship(e);
						m_Callback.Toolbar_Reset();
						break;
					case botao_toolbar.Zoom_Inc:
						Click_Zoom_Inc(e);
						m_Callback.Toolbar_Reset();
						break;
					case botao_toolbar.Zoom_Dec:
						Click_Zoom_Dec(e);
						m_Callback.Toolbar_Reset();
						break;
					case botao_toolbar.Zoom_Clear:
						Click_Zoom_Clear(e);
						m_Callback.Toolbar_Reset();
						break;
					case botao_toolbar.Add_Identifying_Relationship:
						break;
					default:
						break;
				}
			}
		}



		/// <summary>Quando o boto esquerdo do mouse  solto, pode ser:
		/// - Fim de um drag (movimentao)
		/// - Fim de um resize (redimensionamento)
		/// </summary>
		void OnMouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			Point pt = RelativeMousePosition(e.X, e.Y);
			if (e.Button == System.Windows.Forms.MouseButtons.Left)
			{
				// Se estavamos movendo, finalizar...
				if (m_fMoving)
				{
					m_Callback.DocControl_DragEnd (pt.X, pt.Y);
					m_PickedBox = null;
					m_fMoving = false;
				}
					// Se estavamos redimensionando, finalizar...
				else if (m_fResizing)
				{
					m_Callback.DocControl_ResizeEnd (pt.X, pt.Y);
					m_PickedBox = null;
					m_fResizing = false;
				}
				return;
			}
			base.Capture = false;
		}


		/// <summary>Quando o usurio move o mouse, se capturamos alguma ao, tratar...</summary>
		void OnMouseMove (object sender, System.Windows.Forms.MouseEventArgs e)
		{
			Point pt = RelativeMousePosition(e.X, e.Y);
			if (base.Capture)
			{
				// Se estivermos movendo algum elemento...
				if (m_fMoving)
					m_Callback.DocControl_DragContinue (pt.X, pt.Y);
					// Se estivermos redimensionando algum elemento...
				else if (m_fResizing)
					m_Callback.DocControl_ResizeContinue (m_PickedBox, pt.X, pt.Y);

				// redesenha o controle
				this.Invalidate ();
			}
		}


		#region "Aes dos controles da barra de ferramentas"
		/// <summary>Clicou na tela com ferramenta de Adicionar Entidade</summary>
		void Click_AddEntity(System.Windows.Forms.MouseEventArgs e)
		{
			Point pt = RelativeMousePosition(e.X, e.Y);
			IBox NewBox = new Entity();
			m_Callback.DocControl_CreateBox (m_Document, NewBox, pt.X, pt.Y);
			return;
		}

		/// <summary>Clicou na tela com ferramenta de Adicionar Non-Identif.Relationship </summary>
		void Click_Add_NonIdentifying_Relationship(System.Windows.Forms.MouseEventArgs e)
		{
			Point pt = RelativeMousePosition(e.X, e.Y);
			ERWon.ILine NewLine = new ERWon.Line();
			NewLine.Document = m_Document;
			m_Callback.DocControl_CreateLine (m_Document, NewLine, pt.X, pt.Y);
			return;
		}

		/// <summary>Clicou na tela com ferramenta de Aumentar Zoom</summary>
		void Click_Zoom_Inc(System.Windows.Forms.MouseEventArgs e)
		{
			Point pt = RelativeMousePosition(e.X, e.Y);
			float oldzoom = this._zoom;
			this._zoom*=2;
			//TODO - nao consegui centralizar aonde o usurio clica. Nao sei mexer no AutoscrollPosition
			this.AutoScrollPosition=new Point(Convert.ToInt32(pt.X * oldzoom), Convert.ToInt32(pt.Y * oldzoom));
			this.Invalidate ();
		}


		/// <summary>Clicou na tela com ferramenta de Diminuir Zoom</summary>
		void Click_Zoom_Dec(System.Windows.Forms.MouseEventArgs e) 
		{
			//TODO - Talvez seja melhor no esperar clique na tela e mexer no zoom direto.
			Point pt = RelativeMousePosition(e.X, e.Y);
			float oldzoom = this._zoom;
			this._zoom/=2;
			this.AutoScrollPosition=new Point(Convert.ToInt32(pt.X * oldzoom), Convert.ToInt32(pt.Y * oldzoom));
			this.Invalidate ();
		}

		/// <summary>Clicou na tela com ferramenta de resetar Zoom</summary>
		void Click_Zoom_Clear(System.Windows.Forms.MouseEventArgs e) 
		{
			//TODO - Talvez seja melhor no esperar clique na tela e mexer no zoom direto.
			Point pt = RelativeMousePosition(e.X, e.Y);
			float oldzoom = this._zoom;
			this._zoom=1.0f;
			this.AutoScrollPosition=new Point(Convert.ToInt32(pt.X * oldzoom), Convert.ToInt32(pt.Y * oldzoom));
			this.Invalidate ();
		}


		/// <summary>Clicou na tela com ferramenta de Seleo</summary>
		void Click_SelectionTool(System.Windows.Forms.MouseEventArgs e)
		{
			Point pt = RelativeMousePosition(e.X, e.Y);

			// Se no estiver apertado nem shift nem ctrl, resetar a seleo atual de elementos
			//bool fReset = !m_fShiftKey && !m_fControlKey;
			bool fReset =	!(
				(Control.ModifierKeys==Keys.Shift)
				||
				(Control.ModifierKeys==Keys.Control)
				);
						

			// Verificar se h Box ou Line aonde o mouse clicou.
			m_PickedBox = GetPicked (m_Document.Boxes, e) as IBox;
			m_PickedLine = GetPicked (m_Document.Lines, e) as ILine;

			// Acabou de clicar, ento no est movendo.
			m_fMoving = false;

			// Selecionou um Box...
			if (m_PickedBox != null)
			{	
				// Foi clicado em algum extremo? Se foi,  um redimensionamento
				Box_Corner corner = m_PickedBox.GetCorner (pt.X, pt.Y);
				if (((DocElement)m_PickedBox).Selected && !corner.Equals(Box_Corner.None))
				{
					m_fResizing = true;
					m_Callback.DocControl_ResizeBegin (m_Document, m_PickedBox, corner, pt.X, pt.Y);
				}
				else
				{
					// Se no for redimensionamento,  (pode ser) movimentao (drag)
					m_fMoving = true;

					// Movimentao pode ser de vrios elementos selecionados, no apenas um...
					m_Callback.DocControl_BoxPicked (m_PickedBox, fReset);

					// Iniciar a Movimentao (Drag)
					m_Callback.DocControl_DragBegin (m_Document, pt.X, pt.Y);
				}

				// Indica se o controle est capturando o mouse
				base.Capture = true;
				return;
			}
			else
			{
				// Se no clicou em elemento nenhum, clicou no documento em si
				m_Callback.DocControl_DocumentPicked (m_Document, fReset);
			}
		}
		#endregion



		/// <summary>
		/// Atualiza os extremos do documento, baseado nos extremos dos elementos.
		/// Serve para as scrollbars.
		/// </summary>
		void Update_Scrollbars ()
		{
			//TODO - quando o elemento passa do topo ou da esquerda, a scroll nao surge... tentei corrigir, nao consegui. TODO
			m_DocumentBounds = new Rectangle(this.Left, this.Top, this.Width, this.Height);
			Rectangle x = new Rectangle(this.Left, this.Top, this.Width, this.Height);

			foreach (IBox ThisBox in m_Document.Boxes)
			{
				m_DocumentBounds.X = Math.Min(m_DocumentBounds.X, ThisBox.Left);
				m_DocumentBounds.Y = Math.Min(m_DocumentBounds.Y, ThisBox.Top);
				m_DocumentBounds.Width = Math.Max(m_DocumentBounds.Width, ThisBox.Right);
				m_DocumentBounds.Height = Math.Max(m_DocumentBounds.Height, ThisBox.Bottom);
			}
//			((Document) this.m_Document).Drag(-m_DocumentBounds.X, -m_DocumentBounds.Y);
//			this.AutoScrollPosition=new Point(this.AutoScrollPosition.X - m_DocumentBounds.X, this.AutoScrollPosition.Y - m_DocumentBounds.Y);
			this.AutoScrollMinSize = new Size(Convert.ToInt32(m_DocumentBounds.Width * _zoom), Convert.ToInt32(m_DocumentBounds.Height * _zoom));
			Rectangle y = m_DocumentBounds;
			//Size y = new Size(Convert.ToInt32(m_DocumentBounds.Width * _zoom), Convert.ToInt32(m_DocumentBounds.Height * _zoom));
			//(this.m_Callback as MainForm).statusBar1.Text=x.X.ToString() + "," + x.Y.ToString() + "," + x.Width.ToString() + "," + x.Height.ToString() + " ... " + 
			//                                              y.X.ToString() + "," + y.Y.ToString() + "," + y.Width.ToString() + "," + y.Height.ToString();
		}




		/// <summary>Manipula as setas do teclado, para mover os objetos selecionados</summary>
		private void OnKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			//m_fControlKey = e.Control;
			//m_fShiftKey = e.Shift;

			const int movesize = 3; // tamanho das movimentaes
			switch(e.KeyCode) 
			{
				case System.Windows.Forms.Keys.Left:
					DragSelected(-movesize, 0); // TODO - mover apenas objetos selecionados
					this.Invalidate ();
					break;
				case System.Windows.Forms.Keys.Right:
					DragSelected(movesize, 0);
					this.Invalidate ();
					break;
				case System.Windows.Forms.Keys.Up:
					DragSelected(0, -movesize);
					this.Invalidate ();
					break;
				case System.Windows.Forms.Keys.Down:
					DragSelected(0, movesize);
					this.Invalidate ();
					break;
				default:
					break;
			}	
		}

		//		private void OnKeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
		//		{
		//			m_fControlKey = e.Control;
		//			m_fShiftKey = e.Shift;			
		//		}

		/// <summary>
		/// As teclas direcionais por padro no so fornecidas aos eventos do UserControl,
		/// e sim tratadas diretamente pelo WinForms. Para trata-las, precisamos
		/// dar um override na funo IsInputKey, para aceitarmos arrow-keys como
		/// sendo teclas de user-input.
		/// </summary>
		//http://www.syncfusion.com/FAQ/WinForms/FAQ_c41c.asp
		//Para tratarmos teclas direcionais...
		protected override bool IsInputKey(Keys key) 
		{ 
			switch(key) 
			{ 
				case Keys.Up: 
				case Keys.Down: 
				case Keys.Right: 
				case Keys.Left: 
					return true; 
			} 
			return base.IsInputKey(key); 
		} 



		/// <summary>Duplo clique em uma entidade abre form de edio</summary>
		/// Atualmente a edio de Entidades  via duplo-clique na entidade.
		/// TODO: Talvez alterar para Context-Menu? Igual ERWin? 
		private void Control_DoubleClick(object sender, EventArgs e)
		{
			m_PickedBox = GetPicked (m_Document.Boxes, PointToClient(MousePosition)) as Entity;
			if (m_PickedBox != null) 
			{
				frmEditEntity frm = new frmEditEntity(m_PickedBox as Entity, (MainForm) m_Callback, m_Document);
				frm.ShowDialog();
				this.Refresh();
			}
		}


		/// <summary>
		/// Os eventos de mouse recebem coordenadas em relao ao controle.
		/// Esta funo considera a posio das scrollbars para transformar
		/// a posio relativa (recebida no evento) em posio absoluta no documento.
		/// </summary>
		private Point RelativeMousePosition(int X, int Y) 
		{
			int iX = Convert.ToInt32(((X - this.AutoScrollPosition.X) / this._zoom));
			int iY = Convert.ToInt32(((Y - this.AutoScrollPosition.Y) / this._zoom));
			return new Point(iX, iY);
		}

		/// <summary>Ao redimensionar controle, redesenhar...</summary>
		protected override void OnResize(EventArgs e)
		{
			this.Invalidate();
			base.OnResize (e);
		}


	}
}
