/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe Entity  a principal classe de shape do aplicativo.
 * Representa uma entidade (tabela).
 ************************************************************************/

using System;
using System.ComponentModel;
using System.Drawing;
using System.Runtime.Serialization;
using System.Data;

namespace ERWon
{
	/// <summary>Representa uma Entidade ER (tabela).</summary>
	[Serializable, DefaultProperty ("TableName")]
	public class Entity : Box, ISerializable
	{

		/// <summary>Nome fisico da Tabela</summary>
		protected string m_TableName;
		[Description ("Physical Table name"), Category ("Data")]
		public string TableName { get { return m_TableName; } set { m_TableName = value; } }


		public PKIndex m_PK;
		public ColumnCollection m_Cols;
		public Index[] m_Indexes;

		/// <summary>Construtor</summary>
		public Entity () : base()
		{
			base.Height = 90;
			base.Width = 80;
			base.Foreground=Color.Black;
			base.Background=Color.White;
			this.TableName = "TableName";

			m_Indexes = new Index[2];
			m_Cols = new ColumnCollection();
			m_Cols.AddColumn( new Column("ID_Table", "Integer", true) );
			m_Cols.AddColumn( new Column("Description", "String", false) );
			m_Cols.AddColumn( new Column("Height", "Double", false) );
			//m_Cols.DelColumn( m_PK.m_Columns[0] );

			m_PK = new PKIndex("PK_1");
			m_PK.AddColumn(m_Cols.m_Columns[0], true);
			m_Indexes[0] = m_PK;

			Index idx = new UniqueIndex("AK_1");
			idx.AddColumn(m_Cols.m_Columns[1], false);
			m_Indexes[1] = idx;


		}
		/// <summary>Construtor Serializao</summary>
		protected Entity (SerializationInfo Info, StreamingContext Context) : base (Info, Context)
		{
			this.TableName = Info.GetString ("TableName");
			m_Cols = Info.GetValue ("m_Cols", typeof(ColumnCollection)) as ColumnCollection;
			m_PK = Info.GetValue ("m_PK", typeof(PKIndex)) as PKIndex;
			// TODO - pq o indexes nao serializa junto a PK?! nao entendi.
			m_Indexes = Info.GetValue ("m_Indexes", typeof(Index[])) as Index[];
		}
		public override void GetObjectData (SerializationInfo Info, StreamingContext Context)
		{
			base.GetObjectData (Info, Context);
			Info.AddValue ("TableName", this.TableName);
			Info.AddValue ("m_Cols", this.m_Cols);
			Info.AddValue ("m_PK", this.m_PK);
			Info.AddValue("m_Indexes", this.m_Indexes);
		}

		public override uint SpecialPoints { get {return 8;} }
		public override Point GetSpecialPoint (uint ConnectionID)
		{
			switch (ConnectionID)
			{
				case 1:
					return new Point (Left, Top);
				case 2:
					return new Point ((Left+Right)/2, Top);
				case 3:
					return new Point (Right, Top);
				case 4:
					return new Point (Right, (Top+Bottom)/2);
				case 5:
					return new Point (Right, Bottom);
				case 6:
					return new Point ((Left+Right)/2, Bottom);
				case 7:
					return new Point (Left, Bottom);
				case 8:
					return new Point (Left, (Top+Bottom)/2);
				default:
					return new Point(0,0);
			}
		}
		
		/// <summary>Desenha o elemento</summary>
		public override void Draw (System.Drawing.Graphics g)
		{
			Brush BackgroundBrush = null;
			Brush TextBrush = null;
			Pen ForegroundPen = null;

			BackgroundBrush = new SolidBrush (m_Background);
			TextBrush = new SolidBrush (m_Foreground);
			ForegroundPen = new Pen (m_Foreground, 2);

			g.FillRectangle (BackgroundBrush, Left, Top, Width, Height);

			//DrawText (g, TextBrush);
			
			//Height=50;
			//Width=50;

			// TODO - oink oink. Isso s deveria ser atualizado quando as colunas ou indices/pk fossem alterados.
			// Criar evento para atualizar isso.
			// TODO - colocar "FK" aonde for FK.
			string PK_Columns="";
			string NPK_Columns = "";
			for (int i=0; i<this.m_PK.m_Columns.GetLength(0); i++)
				PK_Columns = PK_Columns + "\n" + this.m_PK.m_Columns[i].ColumnName;
			PK_Columns = PK_Columns.TrimStart();
			for (int i=0; i<this.m_Cols.m_Columns.GetLength(0); i++)
				for (int j=0; j<this.m_PK.m_Columns.GetLength(0); j++)
					if (!(this.m_PK.m_Columns[j].ColumnName.Equals(this.m_Cols.m_Columns[i].ColumnName)))
						NPK_Columns = NPK_Columns + "\n" + this.m_Cols.m_Columns[i].ColumnName;
			NPK_Columns = NPK_Columns.TrimStart();


			// TITULO DA TABELA (Retangulo invisivel)
			// tirei o tamanho do width.. deixa ficar maior mesmo.
			RectangleF titulo=new RectangleF(new PointF(Left,Top-20), new SizeF(Width+100, 20));

			// PK (Retangulo)
			Size PK_size = new Size(this.Width, g.MeasureString (PK_Columns, ms_Font).ToSize().Height);
			Rectangle pk = new Rectangle(new Point(Left, Top), PK_size);
			// testar se estourou limite do elemento..
			pk.Height = Math.Min(pk.Height, this.Height);

			// Outras colunas, no-PK (Retangulo)
			// ajustando espao que sobrou do elemento...
			//Size NPK_size = new Size(this.Width, g.MeasureString (NPK_Columns, ms_Font).ToSize().Height);
			Size NPK_size = new Size(this.Width, this.Height - pk.Height);
			Rectangle npk = new Rectangle(new Point(Left, Top + pk.Height), NPK_size);
			// testar se estourou limite do elemento..
			npk.Height = Math.Min(npk.Height, this.Height - pk.Height);

			StringFormat format = new StringFormat (StringFormatFlags.NoWrap);

			// Desenhando efetivamente...
			g.DrawString(m_TableName, ms_Font, TextBrush, titulo, format);
			if (pk.Height > 0) 
			{
				g.DrawRectangle (ForegroundPen, pk);
				g.DrawString(PK_Columns, ms_Font, TextBrush, pk, format);
			}
			if (npk.Height > 0) 
			{
				g.DrawRectangle (ForegroundPen, npk);
				g.DrawString(NPK_Columns, ms_Font, TextBrush, npk, format);
			}


			// Elementos selecionados devem desenhar os corners para redimensionar..
			if (Selected)
			{
				DrawSpecialPoints (g);
				const int FocusMargin = 5;
				Pen FocusPen = new Pen (Util.Highlight (Document.Background), 1);
				g.DrawRectangle (FocusPen, Left - FocusMargin - 1, Top - FocusMargin - 1, Width + FocusMargin + FocusMargin, Height + FocusMargin + FocusMargin);
			}
		}

		/// <summary>Copia para operaes de clipboard</summary>
		public override DocElement CreateClipboardCopy ()
		{
			Entity Copy = base.CreateClipboardCopy () as Entity;
			Copy.TableName = this.TableName;
			Copy.m_Cols = new ColumnCollection();
			for (int i=0; i < this.m_Cols.m_Columns.GetLength(0); i++)
				Copy.m_Cols.AddColumn(new Column(this.m_Cols.m_Columns[i].ColumnName, this.m_Cols.m_Columns[i].typeName, this.m_Cols.m_Columns[i].NOTNULL));
			// TODO - a pk  pra manter referencias e tal... igual clipboard.
//			Copy.m_PK = new PKIndex("PK_1");
//			for (int i=0; i < this.m_PK.m_Columns.GetLength(0); i++)
//				Copy.m_PK.AddColumn(new Column(this.m_PK.m_Columns[i].ColumnName, this.m_PK.m_Columns[i].typeName, this.m_PK.m_Columns[i].NOTNULL));
//			// TODO - indices tbm.
//			Copy.m_Indexes = new ColumnCollection[2];
			//TODO - d pra usar o DuplicateArrayList...

			return Copy as DocElement;
		}




	}
}

