/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe Util
 * Funes utilitrias diversas
 ************************************************************************/

using System;
using System.Drawing;

namespace ERWon
{
	/// <summary>Funes utilitrias...</summary>
	public class Util
	{
		/// <summary>
		/// Serializao de cores: http://www.codetools.com/soap/xmlsettings.asp
		/// </summary>
		public enum ColorFormat
		{
			NamedColor,
			ARGBColor
		}
		public static string ColorToString (Color color)
		{
			if( color.IsNamedColor )
				return string.Format("{0}:{1}", 
					ColorFormat.NamedColor, color.Name);
			else
				return string.Format("{0}:{1}:{2}:{3}:{4}", 
					ColorFormat.ARGBColor, 
					color.A, color.R, color.G, color.B);
		}

		public static Color StringToColor (string strColor)
		{
			byte a, r, g, b;

			string [] pieces = strColor.Split(new char[] {':'});
		
			ColorFormat colorType = (ColorFormat) 
				Enum.Parse(typeof(ColorFormat), pieces[0], true);

			switch (colorType)
			{
				case ColorFormat.NamedColor:
					return Color.FromName(pieces[1]);

				case ColorFormat.ARGBColor:
					a = byte.Parse(pieces[1]);
					r = byte.Parse(pieces[2]);
					g = byte.Parse(pieces[3]);
					b = byte.Parse(pieces[4]);
			
					return Color.FromArgb(a, r, g, b);
			}
			return Color.White;
		}

		/// <summary>Dada uma cor, retorna uma cor contrastante para destaque.</summary>
		public static Color Highlight (Color Normal)
		{
			byte R = (byte) ((Normal.R + 170) % 255);
			byte G = (byte) ((Normal.G + 170) % 255);
			byte B = (byte) ((Normal.B + 170) % 255);
			return Color.FromArgb (R, G, B);
		}

		public const int hittest_precision = 2;

		/// <summary>Determina se as coordenadas dadas esto prximas ao ponto</summary>
		public static bool HitTest (Point ThisPoint, int X, int Y)
		{
			return ((Math.Abs(ThisPoint.X - X) < hittest_precision)
						&&
					(Math.Abs(ThisPoint.Y - Y) < hittest_precision));
		}
	}
}
