/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe CommentBox - Boxes com comentrios. Para anotaes quaisquer no
 * documento.
 ************************************************************************/

using System;
using System.Drawing;
using System.Runtime.Serialization;

namespace ERWon
{
	/// <summary>Boxes com comentrios. Para anotaes quaisquer no documento.</summary>
	[Serializable]
	public class CommentBox : Box
	{
		public CommentBox ()
		{
			m_Foreground = Color.Black;
			m_Background = Color.White;
		}
		protected CommentBox (SerializationInfo Info, StreamingContext Context) : base (Info, Context)
		{
			m_Foreground = Color.Black;
			m_Background = Color.White;
		}

		public override void GetObjectData (SerializationInfo Info, StreamingContext Context)
		{
			base.GetObjectData (Info, Context);
		}
		

		/// <summary>Desenha o elemento</summary>
		public override void Draw (System.Drawing.Graphics g)
		{
			SizeToFitText (g,"X");

			Brush BackgroundBrush = null;
			Brush TextBrush = null;
			Pen ForegroundPen = null;

			BackgroundBrush = new SolidBrush (m_Background);
			TextBrush = new SolidBrush (m_Foreground);
			ForegroundPen = new Pen (m_Foreground, 2);
	
			g.FillRectangle (BackgroundBrush, Left, Top, Width, Height);
			g.DrawRectangle (ForegroundPen, Left, Top, Width, Height);

			DrawText (g, TextBrush);

			if (Selected)
			{
				const int FocusMargin = 5;
				Pen FocusPen = new Pen (Util.Highlight (Document.Background), 1);
				g.DrawRectangle (FocusPen, Left - FocusMargin - 1, Top - FocusMargin - 1, Width + FocusMargin + FocusMargin, Height + FocusMargin + FocusMargin);
			}
		}

		/// <summary>Cria cpia do elemento para clipboard</summary>
		public override DocElement CreateClipboardCopy ()
		{
			CommentBox Copy = base.CreateClipboardCopy () as CommentBox;
			return Copy as DocElement;
		}

		/// <summary>
		/// CommentBoxes no tem SpecialPoints.
		/// Seu tamanho  definido pelo seu contedo (texto).
		/// </summary>
		public override Point GetSpecialPoint (uint ConnectionID) {return new Point(0,0);}
		public override uint SpecialPoints { get {return 0;} }

	}
}