/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe MainForm
 * Janela principal do Aplicativo
 ************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization.Formatters.Soap;

namespace ERWon
{
	/// <summary>Janela principal do aplicativo</summary>
	public class MainForm : System.Windows.Forms.Form, IControlCallbacks
	{

		/// <summary>Documentos abertos</summary>
		private System.Collections.ArrayList m_Documents = new ArrayList ();

		/// <summary>Elementos selecionados (seleo atual)</summary>
		public DocSubset m_SelectionSet;

		public Drag m_CurrentDrag;
		public Resize m_CurrentResize;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.MenuItem menuItem5;
		private System.Windows.Forms.ToolBarButton SelectionTool;
		private System.Windows.Forms.ToolBarButton Add_Entity;
		private System.Windows.Forms.ToolBarButton Add_NonIdentifying_Relationship;
		private System.Windows.Forms.ToolBarButton Add_Identifying_Relationship;
		private System.Windows.Forms.ToolBarButton Add_Comment;
		private System.Windows.Forms.ToolBarButton Zoom_Inc;
		private System.Windows.Forms.ToolBarButton Zoom_Dec;
		private System.Windows.Forms.ToolBarButton Zoom_Clear;

		protected int botao_apertado;
		public botao_toolbar BotaoSelecionado() 
		{
			switch(botao_apertado)
			{
				case 0:
					return botao_toolbar.SelectionTool;
				case 1:
					return botao_toolbar.Add_Entity;
				case 2:
					return botao_toolbar.Add_NonIdentifying_Relationship;
				case 3:
					return botao_toolbar.Add_Identifying_Relationship;
				case 4:
					return botao_toolbar.Add_Comment;
				case 5:
					return botao_toolbar.Zoom_Inc;
				case 6:
					return botao_toolbar.Zoom_Dec;
				case 7:
					return botao_toolbar.Zoom_Clear;
				default:
					return botao_toolbar.None;
			}
		}

		/// <summary>Processador de comandos (Undo/Redo)</summary>
		private CommandProcessor m_CommandProcessor = new CommandProcessor ();


		// Titulo do aplicativo
		const string m_strCaption = "ERWon";

		// Filtro de dialogos de abrir/salvar arquivos 
		const string m_strFileFilter = "ERWon Diagram Files (*.erd)|*.erd|All files (*.*)|*.*";


		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.ToolBar toolBar1;
		public System.Windows.Forms.StatusBar statusBar1;
		private System.Windows.Forms.ImageList botoes_toolbar;
		private System.Windows.Forms.MenuItem File;
		private System.Windows.Forms.MenuItem New;
		private System.Windows.Forms.MenuItem Open;
		private System.Windows.Forms.MenuItem Save;
		private System.Windows.Forms.MenuItem SaveAs;
		private System.Windows.Forms.MenuItem Exit;
		private System.Windows.Forms.MenuItem Edit;
		private System.Windows.Forms.MenuItem Undo;
		private System.Windows.Forms.MenuItem Redo;
		private System.Windows.Forms.MenuItem Cut;
		private System.Windows.Forms.MenuItem Copy;
		private System.Windows.Forms.MenuItem Paste;
		private System.Windows.Forms.MenuItem Delete;
		private System.Windows.Forms.MenuItem SelectAll;
		private System.Windows.Forms.MenuItem Window;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Splitter splitter1;
		private System.Windows.Forms.PropertyGrid m_Properties;
		private System.ComponentModel.IContainer components;

		public MainForm()
		{
			InitializeComponent();

			// Pra alterar o icone do aplicativo foi meio complicado... mas t a.
			//http://www.syncfusion.com/FAQ/WinForms/FAQ_c95c.asp#q774q
			// melhorei... Windows Forms Programming in C#, cap 10.1
			Bitmap bmp = this.botoes_toolbar.Images[9] as Bitmap; 
			//Bitmap bmp = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("ERWon.imagens." + "02-App.ico"));
			this.Icon = Icon.FromHandle(bmp.GetHicon());

			// Criar o subconjunto de elementos selecionados
			m_SelectionSet = new DocSubset (m_Properties);
		}

		public System.Windows.Forms.ImageList Get_Toolbar() 
		{
			return this.toolBar1.ImageList;
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.File = new System.Windows.Forms.MenuItem();
			this.New = new System.Windows.Forms.MenuItem();
			this.Open = new System.Windows.Forms.MenuItem();
			this.Save = new System.Windows.Forms.MenuItem();
			this.SaveAs = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			this.Exit = new System.Windows.Forms.MenuItem();
			this.Edit = new System.Windows.Forms.MenuItem();
			this.Undo = new System.Windows.Forms.MenuItem();
			this.Redo = new System.Windows.Forms.MenuItem();
			this.menuItem5 = new System.Windows.Forms.MenuItem();
			this.Cut = new System.Windows.Forms.MenuItem();
			this.Copy = new System.Windows.Forms.MenuItem();
			this.Paste = new System.Windows.Forms.MenuItem();
			this.Delete = new System.Windows.Forms.MenuItem();
			this.SelectAll = new System.Windows.Forms.MenuItem();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.Window = new System.Windows.Forms.MenuItem();
			this.toolBar1 = new System.Windows.Forms.ToolBar();
			this.SelectionTool = new System.Windows.Forms.ToolBarButton();
			this.Add_Entity = new System.Windows.Forms.ToolBarButton();
			this.Add_NonIdentifying_Relationship = new System.Windows.Forms.ToolBarButton();
			this.Add_Identifying_Relationship = new System.Windows.Forms.ToolBarButton();
			this.Add_Comment = new System.Windows.Forms.ToolBarButton();
			this.Zoom_Inc = new System.Windows.Forms.ToolBarButton();
			this.Zoom_Dec = new System.Windows.Forms.ToolBarButton();
			this.Zoom_Clear = new System.Windows.Forms.ToolBarButton();
			this.botoes_toolbar = new System.Windows.Forms.ImageList(this.components);
			this.statusBar1 = new System.Windows.Forms.StatusBar();
			this.panel1 = new System.Windows.Forms.Panel();
			this.m_Properties = new System.Windows.Forms.PropertyGrid();
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.panel1.SuspendLayout();
			this.SuspendLayout();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.File,
																					  this.Edit,
																					  this.menuItem1,
																					  this.Window});
			// 
			// File
			// 
			this.File.Index = 0;
			this.File.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				 this.New,
																				 this.Open,
																				 this.Save,
																				 this.SaveAs,
																				 this.menuItem4,
																				 this.Exit});
			this.File.Text = "File";
			// 
			// New
			// 
			this.New.Index = 0;
			this.New.Shortcut = System.Windows.Forms.Shortcut.CtrlN;
			this.New.Text = "New";
			this.New.Click += new System.EventHandler(this.FileNew_Click);
			// 
			// Open
			// 
			this.Open.Index = 1;
			this.Open.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.Open.Text = "Open";
			this.Open.Click += new System.EventHandler(this.FileOpen_Click);
			// 
			// Save
			// 
			this.Save.Index = 2;
			this.Save.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
			this.Save.Text = "Save";
			this.Save.Click += new System.EventHandler(this.FileSave_Click);
			// 
			// SaveAs
			// 
			this.SaveAs.Index = 3;
			this.SaveAs.Text = "Save As";
			this.SaveAs.Click += new System.EventHandler(this.FileSaveAs_Click);
			// 
			// menuItem4
			// 
			this.menuItem4.Index = 4;
			this.menuItem4.Text = "-";
			// 
			// Exit
			// 
			this.Exit.Index = 5;
			this.Exit.Shortcut = System.Windows.Forms.Shortcut.AltF4;
			this.Exit.Text = "Exit";
			this.Exit.Click += new System.EventHandler(this.Exit_Click);
			// 
			// Edit
			// 
			this.Edit.Index = 1;
			this.Edit.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				 this.Undo,
																				 this.Redo,
																				 this.menuItem5,
																				 this.Cut,
																				 this.Copy,
																				 this.Paste,
																				 this.Delete,
																				 this.SelectAll});
			this.Edit.Text = "Edit";
			this.Edit.Popup += new System.EventHandler(this.MenuEdit_Popup);
			// 
			// Undo
			// 
			this.Undo.Index = 0;
			this.Undo.Shortcut = System.Windows.Forms.Shortcut.CtrlZ;
			this.Undo.Text = "Undo";
			this.Undo.Click += new System.EventHandler(this.EditUndo_Click);
			// 
			// Redo
			// 
			this.Redo.Index = 1;
			this.Redo.Shortcut = System.Windows.Forms.Shortcut.CtrlY;
			this.Redo.Text = "Redo";
			this.Redo.Click += new System.EventHandler(this.EditRedo_Click);
			// 
			// menuItem5
			// 
			this.menuItem5.Index = 2;
			this.menuItem5.Text = "-";
			// 
			// Cut
			// 
			this.Cut.Index = 3;
			this.Cut.Shortcut = System.Windows.Forms.Shortcut.CtrlX;
			this.Cut.Text = "Cut";
			this.Cut.Click += new System.EventHandler(this.EditCut_Click);
			// 
			// Copy
			// 
			this.Copy.Index = 4;
			this.Copy.Shortcut = System.Windows.Forms.Shortcut.CtrlC;
			this.Copy.Text = "Copy";
			this.Copy.Click += new System.EventHandler(this.EditCopy_Click);
			// 
			// Paste
			// 
			this.Paste.Index = 5;
			this.Paste.Shortcut = System.Windows.Forms.Shortcut.CtrlV;
			this.Paste.Text = "Paste";
			this.Paste.Click += new System.EventHandler(this.EditPaste_Click);
			// 
			// Delete
			// 
			this.Delete.Index = 6;
			this.Delete.Shortcut = System.Windows.Forms.Shortcut.Del;
			this.Delete.Text = "Delete";
			this.Delete.Click += new System.EventHandler(this.EditDelete_Click);
			// 
			// SelectAll
			// 
			this.SelectAll.Index = 7;
			this.SelectAll.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
			this.SelectAll.Text = "SelectAll";
			this.SelectAll.Click += new System.EventHandler(this.EditSelectAll_Click);
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 2;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem2,
																					  this.menuItem3});
			this.menuItem1.Text = "Database";
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 0;
			this.menuItem2.Text = "Import";
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 1;
			this.menuItem3.Text = "Export";
			// 
			// Window
			// 
			this.Window.Index = 3;
			this.Window.MdiList = true;
			this.Window.Text = "Window";
			// 
			// toolBar1
			// 
			this.toolBar1.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																						this.SelectionTool,
																						this.Add_Entity,
																						this.Add_NonIdentifying_Relationship,
																						this.Add_Identifying_Relationship,
																						this.Add_Comment,
																						this.Zoom_Inc,
																						this.Zoom_Dec,
																						this.Zoom_Clear});
			this.toolBar1.DropDownArrows = true;
			this.toolBar1.ImageList = this.botoes_toolbar;
			this.toolBar1.Location = new System.Drawing.Point(0, 0);
			this.toolBar1.Name = "toolBar1";
			this.toolBar1.ShowToolTips = true;
			this.toolBar1.Size = new System.Drawing.Size(648, 38);
			this.toolBar1.TabIndex = 1;
			this.toolBar1.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.ToolBar_Click);
			// 
			// SelectionTool
			// 
			this.SelectionTool.ImageIndex = 0;
			this.SelectionTool.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.SelectionTool.ToolTipText = "Selection";
			// 
			// Add_Entity
			// 
			this.Add_Entity.ImageIndex = 1;
			this.Add_Entity.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.Add_Entity.ToolTipText = "Entity";
			// 
			// Add_NonIdentifying_Relationship
			// 
			this.Add_NonIdentifying_Relationship.ImageIndex = 2;
			this.Add_NonIdentifying_Relationship.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.Add_NonIdentifying_Relationship.ToolTipText = "Non-Identifying Relationship";
			// 
			// Add_Identifying_Relationship
			// 
			this.Add_Identifying_Relationship.ImageIndex = 3;
			this.Add_Identifying_Relationship.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.Add_Identifying_Relationship.ToolTipText = "Identifying Relationship";
			// 
			// Add_Comment
			// 
			this.Add_Comment.ImageIndex = 4;
			// 
			// Zoom_Inc
			// 
			this.Zoom_Inc.ImageIndex = 5;
			// 
			// Zoom_Dec
			// 
			this.Zoom_Dec.ImageIndex = 6;
			// 
			// Zoom_Clear
			// 
			this.Zoom_Clear.ImageIndex = 7;
			// 
			// botoes_toolbar
			// 
			this.botoes_toolbar.ImageSize = new System.Drawing.Size(26, 26);
			this.botoes_toolbar.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("botoes_toolbar.ImageStream")));
			this.botoes_toolbar.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// statusBar1
			// 
			this.statusBar1.Location = new System.Drawing.Point(0, 443);
			this.statusBar1.Name = "statusBar1";
			this.statusBar1.Size = new System.Drawing.Size(648, 16);
			this.statusBar1.TabIndex = 2;
			// 
			// panel1
			// 
			this.panel1.Controls.Add(this.m_Properties);
			this.panel1.Dock = System.Windows.Forms.DockStyle.Left;
			this.panel1.Location = new System.Drawing.Point(0, 38);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(200, 405);
			this.panel1.TabIndex = 4;
			// 
			// m_Properties
			// 
			this.m_Properties.CommandsVisibleIfAvailable = true;
			this.m_Properties.Dock = System.Windows.Forms.DockStyle.Fill;
			this.m_Properties.LargeButtons = false;
			this.m_Properties.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.m_Properties.Location = new System.Drawing.Point(0, 0);
			this.m_Properties.Name = "m_Properties";
			this.m_Properties.Size = new System.Drawing.Size(200, 405);
			this.m_Properties.TabIndex = 0;
			this.m_Properties.Text = "propertyGrid1";
			this.m_Properties.ViewBackColor = System.Drawing.SystemColors.Window;
			this.m_Properties.ViewForeColor = System.Drawing.SystemColors.WindowText;
			this.m_Properties.PropertyValueChanged += new System.Windows.Forms.PropertyValueChangedEventHandler(this.m_Properties_PropertyValueChanged);
			// 
			// splitter1
			// 
			this.splitter1.Location = new System.Drawing.Point(200, 38);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(3, 405);
			this.splitter1.TabIndex = 5;
			this.splitter1.TabStop = false;
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(648, 459);
			this.Controls.Add(this.splitter1);
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.statusBar1);
			this.Controls.Add(this.toolBar1);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.IsMdiContainer = true;
			this.Menu = this.mainMenu1;
			this.Name = "MainForm";
			this.Text = "ERWon";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.panel1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion


		/// <summary>Quando o usurio seleciona um Box no UserControl</summary>
		/// <param name="Picked">O Box selecionado</param>
		/// <param name="fResetSelectionSet">Se  para limpar o conjunto selecionado antes de selecionar (se ctrl e shift no estiverem apertados).</param>
		/// TODO - o BoxPicked, LinePicked, e DocumentPicked esto bem parecidos. Redesenhar?
		public void DocControl_BoxPicked (IBox Picked, bool fResetSelectionSet)
		{
			if (fResetSelectionSet)
			{
				m_SelectionSet.Empty ();
				m_SelectionSet.Select (Picked as Box);
			}
			else
			{
				Box Toggle = Picked as Box;
				if (Toggle != null) 
				{
					if (Toggle.Selected)
					{
						m_SelectionSet.Deselect (Toggle);
					}
					else
					{
						m_SelectionSet.Select (Toggle);
					}
				}
			}			
			GetActiveDocument ().UpdateAllViews ();
		}

		/// <summary>Quando o usurio seleciona uma Line no UserControl</summary>
		/// <param name="Picked">A Line selecionada</param>
		/// <param name="fResetSelectionSet">Se  para limpar o conjunto selecionado antes de selecionar (se ctrl e shift no estiverem apertados).</param>
		public void DocControl_LinePicked (ILine Picked, bool fResetSelectionSet)
		{
			if (fResetSelectionSet)
			{
				m_SelectionSet.Empty ();
				m_SelectionSet.Select (Picked as Line);
			}
			else
			{
				Line Toggle = Picked as Line;
				if (Toggle.Selected)
				{
					m_SelectionSet.Deselect (Toggle);
				}
				else
				{
					m_SelectionSet.Select (Toggle);
				}
			}
			GetActiveDocument ().UpdateAllViews ();
		}

		/// <summary>Quando o usurio seleciona o prprio documento (clicando no fundo) no UserControl</summary>
		/// <param name="document">O documento selecionado</param>
		/// <param name="fResetSelectionSet">Se  para limpar o conjunto selecionado antes de selecionar (se ctrl e shift no estiverem apertados).</param>
		public void DocControl_DocumentPicked (IDocument document, bool fResetSelectionSet)
		{
			if (fResetSelectionSet) 
			{
				m_SelectionSet.Empty ();
				m_SelectionSet.Select (document as DocElement);
			} 
			else 
			{
				if (document.Selected)
				{
					m_SelectionSet.Deselect (document as DocElement);
				}
				else
				{
					m_SelectionSet.Select (document as DocElement);
				}
			}
			GetActiveDocument ().UpdateAllViews ();
		}

		/// <summary>Quando o usurio clica com o boto esquerdo no documento
		/// inicia-se uma operao de Drag, inicializada com a posio atual
		/// do mouse e os elementos selecionados.
		/// </summary>
		public void DocControl_DragBegin (IDocument document, int iStartX, int iStartY)
		{
			// Adiciona os Boxes e Lines selecionados.
			object[] Selected = m_Properties.SelectedObjects;
			ArrayList Dragging = new ArrayList ();
			foreach (object EachThing in Selected)
			{
				DocElement EachObject = EachThing as DocElement;
				if (EachObject == null)
					continue;
				if (EachObject.Document == document)
					Dragging.Add (EachObject);
			}

//			// Se no for nada selecionado, vamos mover o documento todo...
//			// TODO - precisa corrigir o m_fMoving em Control (MouseDown), pra capturar isso..
//			if (Dragging.Count == 0)
//			{
//				Dragging.AddRange (document.Boxes);
//				Dragging.AddRange (document.Lines);
//			}

			m_CurrentDrag = new Drag (document as Document, Dragging, iStartX, iStartY);
		}

		/// <summary>Chamado sempre que o mouse se move e Boxes/Lines esto em
		/// operao de movimentao.</summary>
		public void DocControl_DragContinue (int iNewX, int iNewY)
		{
			if (m_CurrentDrag == null)
				return;
			m_CurrentDrag.Continue (iNewX, iNewY);
		}

		/// <summary>Finalizao de uma operao de Drag(Movimentao).
		/// Salva o deslocamento para poder desfazer a operao.
		/// </summary>
		public void DocControl_DragEnd (int iNewX, int iNewY)
		{
			if (m_CurrentDrag == null)
				return;
			/// Se o drag foi usado (movimentou mesmo), atualizar UI,
			/// e atualizar CommandProcessor.
			if (m_CurrentDrag.Used)
			{
				m_CurrentDrag.Continue (iNewX, iNewY);
				m_CommandProcessor.Execute (m_CurrentDrag);
			}
			m_CurrentDrag = null;
			Refresh_PropertyGrid();
		}

		/// <summary>Quando o usurio clica em um BoxCorner de um box -
		/// pontos especiais para o redimensionamento.
		/// Inicializa uma operao de redimensionamento.
		/// </summary>
		public void DocControl_ResizeBegin (IDocument document, IBox Box, Box_Corner BoxCorner, int iStartX, int iStartY)
		{			
			m_CurrentResize = new Resize (document as Document, Box as Box, BoxCorner, iStartX, iStartY);
		}

		/// <summary>Chamado sempre que o mouse se move e o Box est em
		/// operao de redimensionamento.</summary>
		/// </summary>
		public void DocControl_ResizeContinue (IBox Box, int iNewX, int iNewY)
		{
			if (m_CurrentResize == null)
				return;
			m_CurrentResize.Continue (iNewX, iNewY);
		}

		/// <summary>Finalizao de uma operao de redimensionamento.
		/// Salva a variao para poder desfazer a operao.
		/// </summary>
		public void DocControl_ResizeEnd (int iNewX, int iNewY)
		{
			if (m_CurrentResize == null)
				return;
			/// Se o resize foi usado (redimensionou mesmo), atualizar UI,
			/// e atualizar CommandProcessor.
			if (m_CurrentResize.Used)
			{
				m_CurrentResize.Continue (iNewX, iNewY);
				m_CommandProcessor.Execute (m_CurrentResize);
			}
			m_CurrentResize = null;
			Refresh_PropertyGrid();
		}

		/// <summary>Cria um novo Box, atravs do CommandProcessor para suportar undo/redo</summary>
		public void DocControl_CreateBox (IDocument Document, IBox NewBox, int X, int Y)
		{
			NewBox.Drag(X-NewBox.X0, Y-NewBox.Y0);
			NewBox.Document = Document;
			AddBox Command = new AddBox (NewBox as Box, this, m_Properties.SelectedObjects);
			m_CommandProcessor.Execute (Command);
		}

		/// <summary>Cria uma nova Line atravs atravs do CommandProcessor para suportar undo/redo</summary>
		public void DocControl_CreateLine (IDocument document, ILine NewILine, int X, int Y)
		{
			Line NewLine = NewILine as Line;
			NewLine.Init (X, Y);
			AddLine Command = new AddLine (NewLine, this, m_Properties.SelectedObjects);
			m_CommandProcessor.Execute (Command);
		}


		/// <summary>Algumas operaes exigem refresh no propertygrid...</summary>
		protected void Refresh_PropertyGrid ()
		{
			if (m_Properties != null)
				m_Properties.Refresh();
		}


		/// <summary>Abre um documento de um Stream (arquivo, geralmente),
		/// deserializa, e chama OpenViewForDocument
		/// </summary>
		private void OpenFromStream (Stream File, string strFilename)
		{
			SoapFormatter Formatter = new SoapFormatter ();
			Object foo = null;
				
			/// Serializao est em constante mudana, ento try-catch evita
			/// campos no encontrados...
			try
			{
				foo = Formatter.Deserialize (File);
			}
			catch
			{
			}

			Document NewDocument = foo as Document;

			if (NewDocument == null)
				return;
				
			NewDocument.strPath = strFilename;
			m_Documents.Add (NewDocument);
			OpenViewForDocument (NewDocument);

			File.Close ();
		}



		/// <summary>Cria uma nova janela (form) MDI Child, e abre o documento nela.</summary>
		/// <param name="Doc"></param>
		private void OpenViewForDocument (Document Doc)
		{
			MdiChild NewChild = new MdiChild (this, Doc);

			NewChild.Notify_CaptionChanged += new MdiChild.CaptionChanged (MdiChild_CaptionChanged);
			NewChild.MdiParent = this;
			NewChild.Show ();
		}


		/// <summary>Busca nas janelas MDI Child se o documento pedido j est aberto,
		/// para evitar o mesmo arquivo aberto duas vezes.
		/// </summary>
		private MdiChild AlreadyOpen (string strPath)
		{
			foreach (MdiChild Child in MdiChildren)
			{
				if (Child.DocElement.ParentDocument().strPath == strPath)
					return Child;
			}
			return null;
		}


		/// <summary>Pergunta ao usurio se quer salvar mudanas no documento.
		/// Invocado diretamente pelos forms MDI Child.
		/// </summary>
		public void PromptSave (MdiChild Closing)
		{
			Closing.Activate ();

			DialogResult Result = MessageBox.Show (this, "You have unsaved changes.\nSave before closing?", "Unsaved changes", MessageBoxButtons.YesNo);
			if (Result == DialogResult.Yes)
				FileSave_Click (null, null);
		}



		/// <summary>Documento ativo (documento do form MDI Child ativo)</summary>
		public Document GetActiveDocument ()
		{
			MdiChild ActiveChild = ActiveMdiChild as MdiChild;
			if (ActiveChild == null)
				return null;
			return ActiveChild.DocElement.ParentDocument();
		}

		/// <summary>Cria um documento novo</summary>
		private void FileNew_Click (object sender, System.EventArgs e)
		{
			Document NewDoc = new Document();
			m_Documents.Add (NewDoc);
			OpenViewForDocument (NewDoc);
		}


		// Menu de Quit..
		private void Exit_Click(object sender, System.EventArgs e)
		{
			this.Close ();
		}


		/// <summary>Quando um form MDI Child  fechado.
		/// Limpa do CommandProcessor os comandos daquele documento,
		/// e remove o documento da lista de documentos abertos do aplicativo.
		/// </summary>
		public void DocumentClosed (Document Closed)
		{
			m_CommandProcessor.DocumentClosed (Closed);
			m_Documents.Remove (Closed);
		}

		/// <summary>
		/// Notifica o MainForm de mudanas nos titulos dos documentos nos MDI Childs.
		/// </summary>
		private void MdiChild_CaptionChanged (MdiChild Sender, string strCaption)
		{
			// Se o documento for o documento ativo, atualiza...
			if (Sender == ActiveMdiChild)
				SetCaption ();
		}

		/// <summary>Atualiza o titulo do MainForm.</summary>
		public void SetCaption ()
		{
			if (ActiveMdiChild == null)
			{
				Text = m_strCaption;
				this.m_Properties.SelectedObject = null;
			}
			else
				Text = m_strCaption + " - " + ActiveMdiChild.Text;;
		}

		/// <summary>Salva o documento ativo. Se o documento ainda no tiver path,
		/// invoca o FileSaveAs_Click
		/// </summary>
		private void FileSave_Click(object sender, System.EventArgs e)
		{
			Document ActiveDocument = GetActiveDocument ();

			if (ActiveDocument == null)
				return;

			if (ActiveDocument.strPath != null)
			{
				SoapFormatter Formatter = new SoapFormatter ();
				Stream OutputStream = System.IO.File.OpenWrite (ActiveDocument.strPath);
				Formatter.Serialize (OutputStream, ActiveDocument);
				OutputStream.Close ();

				ActiveDocument.Changed = false;
				SetCaption ();
			}
			else
			{
				FileSaveAs_Click (sender, e);
			}
		}

		/// <summary>Pede um caminho/nome, e salva o documento</summary>
		private void FileSaveAs_Click(object sender, System.EventArgs e)
		{
			// Obtem o documento ativo.
			Document ActiveDocument = GetActiveDocument ();
			if (ActiveDocument == null)
				return;

			// Pede o caminho/nome
			SaveFileDialog SaveAsDlg = new SaveFileDialog (); 
			SaveAsDlg.Filter = m_strFileFilter;
			SaveAsDlg.FilterIndex = 1;
			SaveAsDlg.RestoreDirectory = true;
	
			if (SaveAsDlg.ShowDialog () == DialogResult.OK)
			{
				// Abre o arquivo e salva o documento l
				Stream File;			
				if ((File = SaveAsDlg.OpenFile ()) != null)
				{
					SoapFormatter Formatter = new SoapFormatter ();
					Formatter.Serialize (File, ActiveDocument);
					File.Close ();

					ActiveDocument.strPath = SaveAsDlg.FileName;
					ActiveDocument.UpdateAllViews ();

					ActiveDocument.Changed = false;
					SetCaption ();
				}
			}		
		}





		/// <summary>
		/// O ponto de partida do aplicativo
		/// Abre uma instancia do prprio MainForm
		/// </summary>
		[STAThread]
		static void Main (string[] args) 
		{
			MainForm frm = new MainForm();

			try
			{
				frm.ShowDialog();
			}
			catch (System.Exception e)
			{
				MessageBox.Show("Erro: " + e.ToString ());
			}
		}


		/// <summary>
		/// Quando alguma nova ferramenta da barra de ferramentas  selecionada,
		/// aperta o boto, desaperta os outros, e salva a ferramenta ativada.
		/// </summary>
		private void ToolBar_Click(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			foreach (System.Windows.Forms.ToolBarButton x in ((System.Windows.Forms.ToolBar)sender).Buttons)
				x.Pushed=false;
			e.Button.Pushed=true;
			botao_apertado=((System.Windows.Forms.ToolBar)sender).Buttons.IndexOf(e.Button);
		}

		/// <summary>Restaura a ferramenta de seleo (Selection-Tool, ou cursor).</summary>
		public void Toolbar_Reset()
		{
			foreach (System.Windows.Forms.ToolBarButton x in this.toolBar1.Buttons)
				x.Pushed=false;
			this.toolBar1.Buttons[0].Pushed=true;
			botao_apertado=0;
		}

		/// <summary>Quando alteramos valor no PropertyGrid, redesenhar o documento...</summary>
		private void m_Properties_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
		{
			GetActiveDocument ().UpdateAllViews ();
		}


		/// <summary>Menu Recortar. Copia os elementos selecionados para clipboard
		/// e os apaga.</summary>
		private void EditCut_Click (object sender, System.EventArgs e)
		{
			// Copiando...
			EditCopy_Click (sender, e);

			// Apagando...
			Document ActiveDocument = GetActiveDocument ();
			EditCut Command = new EditCut (ActiveDocument, m_SelectionSet);
			m_CommandProcessor.Execute (Command);
		}

		/// <summary>Menu Copiar. Copia os elementos selecionados para o clipboard</summary>
		private void EditCopy_Click (object sender, System.EventArgs e)
		{
			Document ActiveDocument = GetActiveDocument ();

			// Cria um array com os duplicados
			ArrayList Data = m_SelectionSet.CreateDuplicates (ActiveDocument);

			// Coloca no clipboard
			System.Windows.Forms.Clipboard.SetDataObject (Data);
		}


		/// <summary>Menu colar. Cola elementos do clipboard no documento.</summary>
		private void EditPaste_Click (object sender, System.EventArgs e)
		{
			/// Lendo clipboard
			IDataObject Data = System.Windows.Forms.Clipboard.GetDataObject ();

			/// Est vazio? H formatos nele?
			string[] astrFormats = Data.GetFormats (true);
			if (astrFormats == null)
				return;

			/// O primeiro formato disponvel no clipboard tem dados?
			/// TODO: Acho que nem precisava disso...
			if (!Data.GetDataPresent (astrFormats[0]))
				return;

			/// Obter (testar se so) os dados em formato de ArrayList
			ArrayList NewObjects = Data.GetData (astrFormats[0]) as ArrayList;
			if (NewObjects == null)
				return;
			/// TODO - talvez verificar o contedo do ArrayList... testar.

			Document ActiveDocument = GetActiveDocument ();
			EditPaste Command = new EditPaste (ActiveDocument, NewObjects);
			m_CommandProcessor.Execute (Command);
		}

		/// <summary>Menu Delete. Apaga os elementos selecionados do documento</summary>
		private void EditDelete_Click (object sender, System.EventArgs e)
		{
			Document ActiveDocument = GetActiveDocument ();
			EditDelete Command = new EditDelete (ActiveDocument, m_SelectionSet);
			m_CommandProcessor.Execute (Command);
		}

		/// <summary>Menu Selecionar Tudo. Seleciona todos elementos do documento</summary>
		private void EditSelectAll_Click (object sender, System.EventArgs e)
		{
			Document ActiveDocument = GetActiveDocument ();
			if (ActiveDocument == null)
				return;
			ActiveDocument.SelectAll (m_SelectionSet);
			ActiveDocument.UpdateAllViews ();
		}


		/// <summary>Menu Undo. Desfaz um comando atravs do CommandProcessor.</summary>
		private void EditUndo_Click(object sender, System.EventArgs e)
		{
			m_CommandProcessor.Undo ();
		}


		/// <summary>Menu Redo. Refaz um comando atravs do CommandProcessor.</summary>
		private void EditRedo_Click(object sender, System.EventArgs e)
		{
			m_CommandProcessor.Redo ();		
		}

		/// <summary>
		/// Atualiza o menu de Edit para indicar quais opes esto disponveis.
		/// Para Undo e Redo, se disponveis, tambm mostra os nomes das
		/// respectivas aes (comandos) que estas opes executariam.
		/// </summary>
		private void MenuEdit_Popup(object sender, System.EventArgs e)
		{
			Undo.Enabled = m_CommandProcessor.CanUndo ();		
			if (Undo.Enabled)
				Undo.Text = "Undo " + m_CommandProcessor.UndoText ();
			else
				Undo.Text = "Undo";

			Redo.Enabled = m_CommandProcessor.CanRedo ();
			if (Redo.Enabled)
				Redo.Text = "Redo " + m_CommandProcessor.RedoText ();
			else
				Redo.Text = "Redo";

			Cut.Enabled = !m_SelectionSet.IsEmpty ();

			Copy.Enabled = !m_SelectionSet.IsEmpty ();

			Paste.Enabled = false;
			IDataObject Data = System.Windows.Forms.Clipboard.GetDataObject ();

			if (Data != null)
			{
				string[] astrFormats = Data.GetFormats (true);
				if (astrFormats != null && astrFormats.Length > 0)
				{
					if (Data.GetDataPresent (astrFormats[0]))
					{
						bool fPaste = Data.GetDataPresent ("System.Collections.ArrayList");

						Paste.Enabled = fPaste;						
					}
				}
			}

			Delete.Enabled = !m_SelectionSet.IsEmpty ();

			SelectAll.Enabled = true;
		}


		/// <summary>Menu abrir. Abre dialog pra escolher arquivo, e abre o arquivo.</summary>
		private void FileOpen_Click(object sender, System.EventArgs e)
		{
			Stream File;			

			OpenFileDialog OpenDlg = new OpenFileDialog (); 
			OpenDlg.Filter = m_strFileFilter;
			OpenDlg.FilterIndex = 1;
			OpenDlg.RestoreDirectory = true;
	
			if (OpenDlg.ShowDialog () == DialogResult.OK)
			{
				MdiChild CurrentChild = AlreadyOpen (OpenDlg.FileName);

				if (CurrentChild != null)
				{
					CurrentChild.Activate ();
					return;
				}

				if ((File = OpenDlg.OpenFile ()) != null)
					OpenFromStream (File, OpenDlg.FileName);
			}			
		}

	}
}
