/*************************************************************************
 * ERWon - Entity Relationship for Windows or Not
 * Ricardo Drizin
 * 
 * Classe MdiChild
 * Janela dos documentos MDI
 * Cada documento aberto  hospedado em um form destes.
 ************************************************************************/

using System.Drawing;

namespace ERWon
{
	/// <summary>
	/// Janela dos documentos filhos MDI
	/// Cada documento aberto  hospedado em um form destes.
	/// </summary>
	public class MdiChild : System.Windows.Forms.Form, IView
	{
		private System.ComponentModel.Container components = null;

		/// <summary>O documento sendo mostrado nesta janela</summary>
		private DocElement m_DocElement;
		public DocElement DocElement
		{
			get { return m_DocElement; }
		}

		/// <summary>A janela principal do aplicativo</summary>
		MainForm m_MainForm;

		/// <summary>O usercontrol que ocupa todo o form MdiChild.</summary>
		private Control m_Control;

		public delegate void CaptionChanged (MdiChild Sender, string strCaption);
		public CaptionChanged Notify_CaptionChanged;


		public MdiChild (MainForm mainForm, DocElement Element)
		{
			// Guarda referencia para form pai.
			m_MainForm = mainForm;

			// Guarda referencia para documento.
			m_DocElement = Element;

			/// Seta esta instancia como sendo a viso do documento
			/// TODO - Quando houver modelo fisico *E* logico,
			/// m_View dever ser uma lista de IViews, e a o MdiChild
			/// poder ser uma viso fsica OU lgica.
			Document document = Element as Document;
			if (document == null)
				return;
			document.m_View=this;

			InitializeComponent();

			// Icone do aplicativo.
			//http://www.syncfusion.com/FAQ/WinForms/FAQ_c95c.asp#q774q
			Bitmap bmp = mainForm.Get_Toolbar().Images[9] as Bitmap; 
			this.Icon = Icon.FromHandle(bmp.GetHicon());  

			// Instanciando o UserControl, e mandando ocupar toda a tela.
			m_Control = new Control (mainForm, document);
			m_Control.Dock = System.Windows.Forms.DockStyle.Fill;
			m_Control.Show ();
			ResumeLayout(false);
			this.Controls.Add (m_Control);

			// Atualizar Titulo do MainForm
			SetCaption ();

			// Pedir ao documento para ser notificado de mudanas nele.
			document.ParentDocument().Notify_Changed += new Document.SetChanged (Document_SetChanged);
			document.ParentDocument().Notify_PathChanged += new Document.SetPath (Document_SetPath);
			this.Activated +=new System.EventHandler(MdiChild_Activated);
		}


		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// MdiChild
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.Window;
			this.ClientSize = new System.Drawing.Size(416, 326);
			this.ForeColor = System.Drawing.SystemColors.Window;
			this.Name = "MdiChild";
			this.Text = "ER Model";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.OnClose);

		}
		#endregion

		/// <summary>Se for notificado de mudanas, atualizar Titulo</summary>
		private void Document_SetChanged (bool fModified)
		{
			SetCaption ();
		}
		/// <summary>Se for notificado de mudanas no path, atualizar Titulo</summary>
		private void Document_SetPath (string Path)
		{
			SetCaption ();
		}

		/// <summary>Atualizar titulo. Baseado no nome e nas mudanas desde
		/// a ltima alterao
		/// </summary>
		public void SetCaption ()
		{
			string NewText;

			if (m_DocElement.ParentDocument().strPath != null)
				NewText = m_DocElement.ParentDocument().strPath;
			else
				NewText = "(new model)";

			if (m_DocElement.ParentDocument().Changed)
			{
				NewText = NewText + "*";
			}

			Text = NewText;

			/// Notificar o MainForm
			if (Notify_CaptionChanged != null) 
				Notify_CaptionChanged (this, Text);
		}


		/// <summary>
		/// Redesenha o MdiChild, no caso invocando para o UserControl se redesenhar.
		/// </summary>
		public void Repaint ()
		{
			m_Control.Invalidate ();
		}

		/// <summary>Notifica o MainForm quando esta janela for fechada</summary>
		private void OnClose(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (m_DocElement.ParentDocument().Changed)
				m_MainForm.PromptSave (this);

			/// TODO: Quando for implementado modelos lgico e fsico, 
			/// o fechamento de um modelo no deve implicar fechamento de outro...
			/// rever.
			m_MainForm.DocumentClosed (m_DocElement.ParentDocument());
		}

		private void MdiChild_Activated(object sender, System.EventArgs e)
		{
			this.Repaint();
		}
	}
}
